<?php

require_once DIR_SYSTEM.'library/VerifoneEcomApi/autoload.php';
use Verifone\Mobile_Detect;
use VerifoneEcomApi\ApiWrapper\Authentication\BasicAuth;
use VerifoneEcomApi\ApiWrapper\Http\ApiClient\Client;
use VerifoneEcomApi\ApiWrapper\Http\SimpleCurl;
use VerifoneEcomApi\ApiWrapper\Regions\Emea;
use VerifoneEcomApi\ApiWrapper\Regions\Na;
use VerifoneEcomApi\ApiWrapper\Regions\NzAu;
use VerifoneEcomApi\ApiWrapper\Settings;

class ModelExtensionPaymentVerifoneHosted extends Model
{
    public $settings;
	private $mapper;
	private $simpleCurl;
	private $client;
	private $auth;

    public function __construct($registry)
    {
	    parent::__construct($registry);
	    $this->load->model('checkout/order');
	    $this->load->model('localisation/order_status');
	    $this->simpleCurl = new SimpleCurl();
	    $this->settings = new Settings();
	    $this->settings->setTest( 'yes' == $this->config->get('payment_verifone_hosted_test') )
	                   ->setRegion( $this->config->get('payment_verifone_hosted_region') )
	                   ->setApiUserId( $this->config->get('payment_verifone_hosted_prod_api_user') )
	                   ->setApiKey( $this->config->get('payment_verifone_hosted_prod_api_key') )
	                   ->setTestApiUserId( $this->config->get('payment_verifone_hosted_test_api_user') )
	                   ->setTestApiKey( $this->config->get('payment_verifone_hosted_test_api_key') )
	                   ->set3dsEnable( 'yes' == $this->config->get('payment_verifone_hosted_3ds') )
	                   ->set3dsContractId( $this->config->get('payment_verifone_hosted_prod_3ds_contract') )
	                   ->setPaymentContractId($this->config->get('payment_verifone_hosted_prod_contract') )
	                   ->setEntityId($this->config->get('payment_verifone_hosted_prod_entity') )
	                   ->setTestEntityId( $this->config->get('payment_verifone_hosted_test_entity') )
	                   ->setTestPaymentContractId( $this->config->get('payment_verifone_hosted_test_contract') )
	                   ->setTest3dsContractId( $this->config->get('payment_verifone_hosted_test_3ds_contract') )
	                   ->setCurlVerifySsl( '1' == $this->config->get('payment_verifone_hosted_curl') )
	                   ->setCvvOnPayment( '1' == $this->config->get('payment_verifone_hosted_cvv') );
	    $this->getRegionMapper();
	    $this->auth = new BasicAuth($this->settings);
	    $this->client = new Client(
		    $this->mapper,
		    $this->settings,
		    $this->auth ,
		    $this->simpleCurl
	    );
    }

	/**
	 *
	 */
	public function getRegionMapper() {
		// mini factory :D
		switch ( $this->config->get('payment_verifone_hosted_region') )
		{
			case 'NZAU':
				$this->mapper = new Nzau( $this->settings );
				break;
			case 'NA':
				$this->mapper = new Na( $this->settings );
				break;
			case 'EMEA':
				$this->mapper = new Emea( $this->settings );
				break;
			default:
				// will happen only on the first installation
				$this->mapper = new Nzau( $this->settings );
		}
	}

    /**
     * @param $params
     * @throws \Exception
     */
    public function updateOrder($params)
    {
        try {
	        $result  =  $this->client->getCheckout($params['checkout_id']);
        } catch (Exception $e) {
            throw new Exception($e->getMessage());
        }
	    if (isset($result['code']) && $result['code'] && $result['message']) {

		    throw new Exception($result['message']);
	    }

	    if (!isset($result['events'])
	        || !isset($result['merchant_reference'])
	        || !isset($result['transaction_id'])
	    ) {

		    throw new Exception('There was an error in the application');
	    }

	    $this->model_checkout_order->addOrderHistory(
		    $result['merchant_reference'],
		    (int)$this->config->get('payment_verifone_hosted_order_status_id'),
		    'Verifone transaction ID: <strong style="color: #0D4AA2">' . $result['transaction_id'] . '</strong>'
	    );
	    $this->addTransaction($result);
    }

    /**
     * once the payment is made we save the transaction for future refunds
     * @param $params
     * @return mixed
     */
    private function addTransaction($params)
    {
        $total = ($params['amount'] / 100); // the amount is in cents so we make it to be normal
        $status = end($params['events'])['type']; // last event matters

        return $this->db->query("
                INSERT INTO `" . DB_PREFIX . "verifone_hosted` SET 
                `order_id` = '" . (int)$params['merchant_reference'] . "', 
                `transaction_id` = '" . $this->db->escape($params['transaction_id']) . "',
                `amount` = '" . $total . "',
                `currency` = '" . $this->db->escape($params['currency_code']) . "',
                `verifone_order_status` = '" . $this->db->escape($status) . "'
                ");
    }


    /**
     * @param $order_id
     * @return mixed
     * @throws \Exception
     */
    public function create($order_id)
    {
	    $order_info = $this->model_checkout_order->getOrder($order_id);
	    $postCustomer = $this->buildCustomer($order_info);

	    try {
		    $customer       =  $this->client->postCustomer($postCustomer);
		 } catch (Exception $e) {
		    throw new Exception($e->getMessage());
	    }
	    if (!isset($customer['id'])) {
		    throw new Exception('Failed to create customer' . (isset($customer['message'])) ? ': ' . $customer['message'] . '.' : '.');
	    }
	    $customer_id = $customer['id'];

	    $total = $this->currency->format($order_info['total'], $order_info['currency_code'], $order_info['currency_value'], false);
	    $order_params = $this->buildOrder($order_info, $customer_id, $total);

		try{
            $result = $this->client->postCheckout($order_params);
        } catch (Exception $e) {
			throw new Exception($e->getMessage());
		}

        $message = 'Something went wrong. Checkout was not created!';
        if (isset($result['code']) || !isset($result['url'])) { //error
            if (isset($result['details']) && isset($result['details']['message']) &&
                !empty($result['details']['message'])) {
                $message = $result['details']['message'];
            }
            $result = [
                'success'   => false,
                'redirect' => null,
                'message'  => $message
            ];
        } elseif (isset($result['url'])) {
            $result = [
                'success'   => true,
                'redirect' => $result['url'],
                'message'  => 'Redirecting to... ' . $result['url']
            ];
            $this->model_checkout_order->addOrderHistory(
                $order_id,
                1, // pending status
                'Waiting for client to finalize the payment',
                false// mail
            );
        } else {
            $result = [
                'success'   => false,
                'redirect' => null,
                'message'  => $message
            ];
        }

        return json_encode($result);
    }

	/**
	 * @return string
	 */
	private function getDeviceType()
	{
		$device_type = 'S';
		$checker = new Mobile_Detect();
		if ($checker->isMobile() || $checker->isTablet()) {
			$device_type = 'P';
		}

		return $device_type;
	}

	public function buildOrder($order_info, $customer_id, $total)
	{

		$params = [
			'customer'          => (string)$customer_id,
			'amount'            => (int)($total * 100), // it needs to be in cents...
			'currency_code'      => strtoupper($order_info['currency_code']),
			'entity_id'          => $this->settings->getEntityId(),
			'merchant_reference' => (string)$order_info['order_id'],
			'return_url'         => $this->url->link('extension/payment/verifone_hosted/callback', '', true),
			'configurations'    => [
				'card' => [
					'capture_now'        => true,
					'cvv_required'       => $this->settings->getCvvOnPayment(),
					'payment_contract_id' => $this->settings->getPaymentContractId(),
				]
			]
		];
		if ($this->settings->is3dsEnabled()) {
			$params['configurations']['card']['threed_secure'] = [
				'threeds_contract_id' => $this->settings->get3dsContractId(),
				'enabled'           => true,
				'transaction_mode'   => $this->getDeviceType(),
			];
		}

		return $params;
	}

	/**
	 * @param $order
	 * @param $type
	 * @return array
	 */
	private function getAddress($order, $type)
	{
		// shipping required but not set ( we get the same as billing)
		if ($type === 'shipping' && empty($order['shipping_address_1'])) {
			return $this->getAddress($order, 'payment');
		}

		return [
			'address_1'    => substr($order[$type . '_address_1'], 0, 40),
			'address_2'    => isset($order[$type . '_address_2']) && !empty($order[$type . '_address_2']) ?
				substr($order[$type . '_address_2'], 0, 40) :
				substr($order['payment_address_1'], 0, 40),
			'city'        => substr($order[$type . '_city'], 0, 50),
			'country_code' => strtoupper($order[$type . '_iso_code_2']),
			'first_name'   => substr($order[$type . '_firstname'], 0, 50),
			'last_name'    => substr($order[$type . '_lastname'], 0, 50),
			'phone'       => $this->formatPhoneNumber($order['telephone']),
			'postal_code'  => substr($order[$type . '_postcode'], 0, 10),
			'state'       => substr($order[$type . '_zone'], 0, 50)
		];
	}


	/**
	 * @param $data
	 * @return false|string
	 */
	private function toJson($data)
	{
		return json_encode($data, JSON_UNESCAPED_UNICODE);
	}

	/**
	 * @param $nr
	 * @return string
	 */
	private function formatPhoneNumber($nr)
	{
		return (string)preg_replace('/\D/', '', $nr);
	}

	private function buildCustomer($order)
	{
		$params = [
			'entity_id'     => $this->settings->getEntityId(),
			'email_address' => substr($order['email'], 0, 255),
			'phone_number'  => $this->formatPhoneNumber($order['telephone']),
			'billing'      => $this->getAddress($order, 'payment'),
			'shipping'     => $this->getAddress($order, 'shipping')
		];

		if ($order['payment_company'] !== '') {
			$params['company_name'] = substr($order['payment_company'], 0, 100);
		}

		return $params;
	}


    /**
     * @param $address
     * @param $total
     * @return array
     */
    public function getMethod($address, $total)
    {
        $this->load->language('extension/payment/verifone_hosted');
        $query = $this->db->query("SELECT * FROM " . DB_PREFIX . "zone_to_geo_zone WHERE geo_zone_id = '" . (int)$this->config->get('payment_verifone_hosted_geo_zone_id') . "' AND country_id = '" . (int)$address['country_id'] . "' AND (zone_id = '" . (int)$address['zone_id'] . "' OR zone_id = '0')");
        if ($this->config->get('payment_verifone_hosted_total') > 0 && $this->config->get('payment_verifone_hosted_total') > $total) {
            $status = false;
        } elseif (!$this->config->get('payment_verifone_hosted_geo_zone_id')) {
            $status = true;
        } elseif ($query->num_rows) {
            $status = true;
        } else {
            $status = false;
        }

        $method_data = [];

        if ($status) {
            $method_data = [
                'code'       => 'verifone_hosted',
                'title'      => $this->language->get('text_title'),
                'terms'      => '',
                'sort_order' => $this->config->get('payment_verifone_hosted_sort_order')
            ];
        }

        return $method_data;
    }


}
